@@ -25,7 +25,7 @@ PPI::Document - Object representation of a Perl document
   # Find all the named subroutines
   my $sub_nodes = $Document->find( 
   	sub { $_[1]->isa('PPI::Statement::Sub') and $_[1]->name }
-  	);
+  );
   my @sub_names = map { $_->name } @$sub_nodes;
   
   # Save the file
@@ -48,8 +48,8 @@ C<PPI::Document> implements the necessary C<STORABLE_freeze> and
 C<STORABLE_thaw> hooks to provide native support for L<Storable>,
 if you have it installed.
 
-However if you want to clone clone a Document, you are highly recommended
-to use the internal C<$Document-E<gt>clone> method rather than Storable's
+However if you want to clone a Document, you are highly recommended
+to use the C<$Document-E<gt>clone> method rather than Storable's
 C<dclone> function (although C<dclone> should still work).
 
 =head1 METHODS
@@ -64,31 +64,33 @@ Document-specific.
 =cut
 
 use strict;
-use base 'PPI::Node';
 use Carp                          ();
-use List::MoreUtils               ();
-use Params::Util                  '_INSTANCE',
-                                  '_SCALAR';
+use List::Util 1.33               ();
+use Params::Util 1.00             qw{_SCALAR0 _ARRAY0 _INSTANCE};
 use Digest::MD5                   ();
-use PPI                           ();
 use PPI::Util                     ();
-use PPI::Exception::ParserTimeout ();
-use overload 'bool'               => sub () { 1 };
-use overload '""'                 => 'content';
-use constant HAS_ALARM            => (
-	$^O eq 'MSWin32' or $^O eq 'cygwin'
-	) ? 0 : 1;
+use PPI                           ();
+use PPI::Node                     ();
+use YAML::PP                      ();
 
-use vars qw{$VERSION $errstr};
-BEGIN {
-	$VERSION = '1.203';
-	$errstr  = '';
-}
+use overload 'bool' => \&PPI::Util::TRUE;
+use overload '""'   => 'content';
+
+our $VERSION = '1.284';
+
+our ( $errstr, @ISA ) = ( "", "PPI::Node" );
 
 use PPI::Document::Fragment ();
 
 # Document cache
-my $CACHE = undef;
+my $CACHE;
+
+# Convenience constants related to constants
+use constant LOCATION_LINE         => 0;
+use constant LOCATION_CHARACTER    => 1;
+use constant LOCATION_COLUMN       => 2;
+use constant LOCATION_LOGICAL_LINE => 3;
+use constant LOCATION_LOGICAL_FILE => 4;
 
 
 
@@ -108,7 +110,7 @@ my $CACHE = undef;
   # With the readonly attribute set
   $doc = PPI::Document->new( $filename,
           readonly => 1,
-          );
+  );
 
 The C<new> constructor takes as argument a variety of different sources of
 Perl code, and creates a single cohesive Perl C<PPI::Document>
@@ -127,16 +129,57 @@ In all cases, the document is considered to be "anonymous" and not tied back
 to where it was created from. Specifically, if you create a PPI::Document from
 a filename, the document will B<not> remember where it was created from.
 
+Returns a C<PPI::Document> object, or C<undef> if parsing fails.
+L<PPI::Exception> objects can also be thrown if there are parsing problems.
+
 The constructor also takes attribute flags.
 
-At this time, the only available attribute is the C<readonly> flag.
+=head3 readonly
 
-Setting C<readonly> to true will allow various systems to provide
-additional optimisations and caching. Note that because C<readonly> is an
-optimisation flag, it is off by default and you will need to explicitly
-enable it.
+Setting C<readonly> to true will allow various systems to provide additional
+optimisations and caching. Note that because C<readonly> is an optimisation
+flag, it is off by default and you will need to explicitly enable it.
 
-Returns a C<PPI::Document> object, or C<undef> if parsing fails.
+=head3 feature_mods
+
+Setting feature_mods with a hashref allows defining perl parsing features to be
+enabled for the whole document. (e.g. when the code is assumed to be run as a
+oneliner)
+
+=head3 custom_feature_includes
+
+  custom_feature_includes =>
+    { strEct => { signatures => "Syntax::Keyword::Try" } }
+
+Setting custom_feature_includes with a hashref allows defining include names
+which act like pragmas that enable parsing features within their scope.
+
+This is mostly useful when your work project has its own boilerplate module.
+
+It can also be provided as JSON or YAML in the environment variable
+PPI_CUSTOM_FEATURE_INCLUDES, like so:
+
+  PPI_CUSTOM_FEATURE_INCLUDES='strEct: {signatures: perl}' \
+    perlcritic lib/OurModule.pm
+
+  PPI_CUSTOM_FEATURE_INCLUDES='{"strEct":{"signatures":"perl"}}' \
+    perlcritic lib/OurModule.pm
+
+=head3 custom_feature_include_cb
+
+  custom_feature_include_cb => sub {
+    my ($statement) = @_;
+    return $statement->module eq "strEct" ? { signatures => "perl" } : ();
+  },
+
+Setting custom_feature_include_cb with a code reference causes all inspections
+on includes to call that sub before doing any other inspections. The sub can
+decide to either return a hashref of features to be enabled or disabled, which
+will be used for the scope the include was called in, or undef to continue with
+the default inspections. The argument to the sub will be the
+L<PPI::Statement::Include> object.
+
+This can be useful when your work project has a complex boilerplate module.
 
 =cut
 
@@ -154,10 +197,6 @@ sub new {
 	# Check constructor attributes
 	my $source  = shift;
 	my %attr    = @_;
-	my $timeout = delete $attr{timeout};
-	if ( $timeout and ! HAS_ALARM ) {
-		Carp::croak("This platform does not support PPI parser timeouts");
-	}
 
 	# Check the data source
 	if ( ! defined $source ) {
@@ -169,72 +208,47 @@ sub new {
 			Carp::croak("API CHANGE: Source code should only be passed to PPI::Document->new as a SCALAR reference");
 		}
 
+		# Save the filename
+		$attr{filename} ||= $source;
+
 		# When loading from a filename, use the caching layer if it exists.
 		if ( $CACHE ) {
-			my $file   = $source;
-			my $source = PPI::Util::_slurp( $file );
-			unless ( ref $source ) {
-				# Errors returned as plain string
-				return $class->_error($source);
-			}
+			my $file_contents = PPI::Util::_slurp( $source );
+
+			# Errors returned as plain string
+			return $class->_error($file_contents) if !ref $file_contents;
 
 			# Retrieve the document from the cache
-			my $document = $CACHE->get_document($source);
+			my $document = $CACHE->get_document($file_contents);
 			return $class->_setattr( $document, %attr ) if $document;
 
-			if ( $timeout ) {
-				eval {
-					local $SIG{ALRM} = sub { die "alarm\n" };
-					alarm( $timeout );
-					$document = PPI::Lexer->lex_source( $$source );
-					alarm( 0 );
-				};
-			} else {
-				$document = PPI::Lexer->lex_source( $$source );
-			}
+			$document = PPI::Lexer->lex_source( $$file_contents, %attr );
 			if ( $document ) {
 				# Save in the cache
 				$CACHE->store_document( $document );
-				return $class->_setattr( $document, %attr );
+				return $document;
 			}
 		} else {
-			if ( $timeout ) {
-				eval {
-					local $SIG{ALRM} = sub { die "alarm\n" };
-					alarm( $timeout );
-					my $document = PPI::Lexer->lex_file( $source );
-					return $class->_setattr( $document, %attr ) if $document;
-					alarm( 0 );
-				};
-			} else {
-				my $document = PPI::Lexer->lex_file( $source );
-				return $class->_setattr( $document, %attr ) if $document;
-			}
+			my $document = PPI::Lexer->lex_file( $source, %attr );
+			return $document if $document;
 		}
 
-	} elsif ( _SCALAR($source) ) {
-		if ( $timeout ) {
-			eval {
-				local $SIG{ALRM} = sub { die "alarm\n" };
-				alarm( $timeout );
-				my $document = PPI::Lexer->lex_source( $$source );
-				return $class->_setattr( $document, %attr ) if $document;
-				alarm( 0 );
-			}
-		} else {
-			my $document = PPI::Lexer->lex_source( $$source );
-			return $class->_setattr( $document, %attr ) if $document;
-		}
+	} elsif ( _SCALAR0($source) ) {
+		my $document = PPI::Lexer->lex_source( $$source, %attr );
+		return $document if $document;
+
+	} elsif ( _ARRAY0($source) ) {
+		$source = join '', map { "$_\n" } @$source;
+		my $document = PPI::Lexer->lex_source( $source, %attr );
+		return $document if $document;
 
 	} else {
-		$class->_error("An unknown object or reference was passed to PPI::Document::new");
+		$class->_error("Unknown object or reference was passed to PPI::Document::new");
 	}
 
 	# Pull and store the error from the lexer
 	my $errstr;
-	if ( _INSTANCE($@, 'PPI::Exception::Timeout') ) {
-		$errstr = 'Timed out while parsing document';
-	} elsif ( _INSTANCE($@, 'PPI::Exception') ) {
+	if ( _INSTANCE($@, 'PPI::Exception') ) {
 		$errstr = $@->message;
 	} elsif ( $@ ) {
 		$errstr = $@;
@@ -253,8 +267,21 @@ sub load {
 }
 
 sub _setattr {
-	my ($class, $document, %attr) = @_;
-	$document->{readonly} = !! $attr{readonly};
+	my ( $class, $document, %attr ) = @_;
+	$document->{readonly}                  = !!$attr{readonly};
+	$document->{filename}                  = $attr{filename};
+	$document->{feature_mods}              = $attr{feature_mods};
+	$document->{custom_feature_includes}   = $attr{custom_feature_includes};
+	$document->{custom_feature_include_cb} = $attr{custom_feature_include_cb};
+	if ( $ENV{PPI_CUSTOM_FEATURE_INCLUDES} ) {
+		my $includes = YAML::PP::Load $ENV{PPI_CUSTOM_FEATURE_INCLUDES};
+		die "\$ENV{PPI_CUSTOM_FEATURE_INCLUDES} "
+		  . "does not contain valid perl:\n"
+		  . "val: '$ENV{PPI_CUSTOM_FEATURE_INCLUDES}'\nerr: $@"
+		  if $@;
+		$document->{custom_feature_includes} =
+		  { %{ $document->{custom_feature_includes} || {} }, %{$includes} };
+	}
 	return $document;
 }
 
@@ -320,6 +347,19 @@ sub get_cache {
 
 =pod
 
+=head2 filename
+
+The C<filename> accessor returns the name of the file in which the document
+is stored.
+
+=cut
+
+sub filename {
+	$_[0]->{filename};
+}
+
+=pod
+
 =head2 readonly
 
 The C<readonly> attribute indicates if the document is intended to be
@@ -357,6 +397,36 @@ sub tab_width {
 	$self->{tab_width} = shift;
 }
 
+=head2 feature_mods { feature_name => $provider }
+
+=cut
+
+sub feature_mods {
+	my $self = shift;
+	return $self->{feature_mods} unless @_;
+	$self->{feature_mods} = shift;
+}
+
+=head2 custom_feature_includes { module_name => { feature_name => $provider } }
+
+=cut
+
+sub custom_feature_includes {
+	my $self = shift;
+	return $self->{custom_feature_includes} unless @_;
+	$self->{custom_feature_includes} = shift;
+}
+
+=head2 custom_feature_include_cb sub { ... }
+
+=cut
+
+sub custom_feature_include_cb {
+	my $self = shift;
+	return $self->{custom_feature_include_cb} unless @_;
+	$self->{custom_feature_include_cb} = shift;
+}
+
 =pod
 
 =head2 save
@@ -373,6 +443,7 @@ sub save {
 	my $self = shift;
 	local *FILE;
 	open( FILE, '>', $_[0] )    or return undef;
+	binmode FILE;
 	print FILE $self->serialize or return undef;
 	close FILE                  or return undef;
 	return 1;
@@ -400,15 +471,15 @@ Returns the serialized document as a string.
 
 sub serialize {
 	my $self   = shift;
-	my @Tokens = $self->tokens;
+	my @tokens = $self->tokens;
 
 	# The here-doc content buffer
 	my $heredoc = '';
 
 	# Start the main loop
 	my $output = '';
-	foreach my $i ( 0 .. $#Tokens ) {
-		my $Token = $Tokens[$i];
+	foreach my $i ( 0 .. $#tokens ) {
+		my $Token = $tokens[$i];
 
 		# Handle normal tokens
 		unless ( $Token->isa('PPI::Token::HereDoc') ) {
@@ -437,12 +508,15 @@ sub serialize {
 
 		# This token is a HereDoc.
 		# First, add the token content as normal, which in this
-		# case will definately not contain a newline.
+		# case will definitely not contain a newline.
 		$output .= $Token->content;
 
+		# Pick up the indentation, which may be undef.
+		my $indentation = $Token->indentation || '';
+
 		# Now add all of the here-doc content to the heredoc buffer.
 		foreach my $line ( $Token->heredoc ) {
-			$heredoc .= $line;
+			$heredoc .= "\n" eq $line ? $line : $indentation . $line;
 		}
 
 		if ( $Token->{_damaged} ) {
@@ -452,21 +526,21 @@ sub serialize {
 			# from the end of a file that we silently allow.
 			#
 			# When writing back out to the file we have to
-			# auto-repair these problems if we arn't going back
+			# auto-repair these problems if we aren't going back
 			# on to the end of the file.
 
 			# When calculating $last_line, ignore the final token if
 			# and only if it has a single newline at the end.
-			my $last_index = $#Tokens;
-			if ( $Tokens[$last_index]->{content} =~ /^[^\n]*\n$/ ) {
+			my $last_index = $#tokens;
+			if ( $tokens[$last_index]->{content} =~ /^[^\n]*\n$/ ) {
 				$last_index--;
 			}
 
 			# This is a two part test.
 			# First, are we on the last line of the
 			# content part of the file
-			my $last_line = List::MoreUtils::none {
-				$Tokens[$_] and $Tokens[$_]->{content} =~ /\n/
+			my $last_line = List::Util::none {
+				$tokens[$_] and $tokens[$_]->{content} =~ /\n/
 				} (($i + 1) .. $last_index);
 			if ( ! defined $last_line ) {
 				# Handles the null list case
@@ -475,14 +549,14 @@ sub serialize {
 
 			# Secondly, are their any more here-docs after us,
 			# (with content or a terminator)
-			my $any_after = List::MoreUtils::any {
-				$Tokens[$_]->isa('PPI::Token::HereDoc')
+			my $any_after = List::Util::any {
+				$tokens[$_]->isa('PPI::Token::HereDoc')
 				and (
-					scalar(@{$Tokens[$_]->{_heredoc}})
+					scalar(@{$tokens[$_]->{_heredoc}})
 					or
-					defined $Tokens[$_]->{_terminator_line}
+					defined $tokens[$_]->{_terminator_line}
 					)
-				} (($i + 1) .. $#Tokens);
+				} (($i + 1) .. $#tokens);
 			if ( ! defined $any_after ) {
 				# Handles the null list case
 				$any_after = '';
@@ -506,7 +580,7 @@ sub serialize {
 
 		# Now add the termination line to the heredoc buffer
 		if ( defined $Token->{_terminator_line} ) {
-			$heredoc .= $Token->{_terminator_line};
+			$heredoc .= $indentation . $Token->{_terminator_line};
 		}
 	}
 
@@ -545,8 +619,7 @@ Returns a 32 character hexadecimal string.
 =cut
 
 sub hex_id {
-	my $self = shift;
-	PPI::Util::md5hex($self->serialize);
+	PPI::Util::md5hex($_[0]->serialize);
 }
 
 =pod
@@ -562,84 +635,166 @@ and very slow if you need to do it a lot. So the C<index_locations> method
 will index and save the locations of every Element within the Document in
 advance, making future calls to <PPI::Element::location> virtually free.
 
-Please note that this is index should always be cleared using
-C<flush_locations> once you are finished with the locations. If content is
-added to or removed from the file, these indexed locations will be B<wrong>.
+Please note that this index should always be cleared using C<flush_locations>
+once you are finished with the locations. If content is added to or removed
+from the file, these indexed locations will be B<wrong>.
 
 =cut
 
 sub index_locations {
 	my $self   = shift;
-	my @Tokens = $self->tokens;
+	my @tokens = $self->tokens;
 
 	# Whenever we hit a heredoc we will need to increment by
-	# the number of lines in it's content section when when we
+	# the number of lines in its content section when we
 	# encounter the next token with a newline in it.
 	my $heredoc = 0;
 
 	# Find the first Token without a location
 	my ($first, $location) = ();
-	foreach ( 0 .. $#Tokens ) {
-		my $Token = $Tokens[$_];
+	foreach ( 0 .. $#tokens ) {
+		my $Token = $tokens[$_];
 		next if $Token->{_location};
 
 		# Found the first Token without a location
 		# Calculate the new location if needed.
-		$location = $_
-			? $self->_add_location( $location, $Tokens[$_ - 1], \$heredoc )
-			: [ 1, 1, 1 ];
+		$location =
+			$_
+		  ? $self->_add_location( $location, $tokens[ $_ - 1 ], \$heredoc )
+		  : $self->_default_location;
 		$first = $_;
 		last;
 	}
 
 	# Calculate locations for the rest
-	foreach ( $first .. $#Tokens ) {
-		my $Token = $Tokens[$_];
-		$Token->{_location} = $location;
-		$location = $self->_add_location( $location, $Token, \$heredoc );
+	if ( defined $first ) {
+		foreach ( $first .. $#tokens ) {
+			my $Token = $tokens[$_];
+			$Token->{_location} = $location;
+			$location = $self->_add_location( $location, $Token, \$heredoc );
 
-		# Add any here-doc lines to the counter
-		if ( $Token->isa('PPI::Token::HereDoc') ) {
-			$heredoc += $Token->heredoc + 1;
+			# Add any here-doc lines to the counter
+			if ( $Token->isa('PPI::Token::HereDoc') ) {
+				$heredoc += $Token->heredoc + 1;
+			}
 		}
 	}
 
 	1;
 }
 
+sub _default_location {
+	my ($self) = @_;
+	my $logical_file = $self->can('filename') ? $self->filename : undef;
+	return [ 1, 1, 1, 1, $logical_file ];
+}
+
+sub location {
+	my ($self) = @_;
+	return $self->SUPER::location || $self->_default_location;
+}
+
 sub _add_location {
 	my ($self, $start, $Token, $heredoc) = @_;
 	my $content = $Token->{content};
 
 	# Does the content contain any newlines
 	my $newlines =()= $content =~ /\n/g;
+	my ($logical_line, $logical_file) =
+		$self->_logical_line_and_file($start, $Token, $newlines);
+
 	unless ( $newlines ) {
 		# Handle the simple case
 		return [
-			$start->[0],
-			$start->[1] + length($content),
-			$start->[2] + $self->_visual_length($content, $start->[2])
+			$start->[LOCATION_LINE],
+			$start->[LOCATION_CHARACTER] + length($content),
+			$start->[LOCATION_COLUMN]
+				+ $self->_visual_length(
+					$content,
+					$start->[LOCATION_COLUMN]
+				),
+			$logical_line,
+			$logical_file,
 		];
 	}
 
 	# This is the more complex case where we hit or
 	# span a newline boundary.
-	my $location = [ $start->[0] + $newlines, 1, 1 ];
+	my $physical_line = $start->[LOCATION_LINE] + $newlines;
+	my $location = [ $physical_line, 1, 1, $logical_line, $logical_file ];
 	if ( $heredoc and $$heredoc ) {
-		$location->[0] += $$heredoc;
+		$location->[LOCATION_LINE]         += $$heredoc;
+		$location->[LOCATION_LOGICAL_LINE] += $$heredoc;
 		$$heredoc = 0;
 	}
 
 	# Does the token have additional characters
 	# after their last newline.
 	if ( $content =~ /\n([^\n]+?)\z/ ) {
-		$location->[1] += length($1);
-		$location->[2] += $self->_visual_length($1, $location->[2]);
+		$location->[LOCATION_CHARACTER] += length($1);
+		$location->[LOCATION_COLUMN] +=
+			$self->_visual_length(
+				$1, $location->[LOCATION_COLUMN],
+			);
 	}
 
 	$location;
 }
 
+sub _logical_line_and_file {
+	my ($self, $start, $Token, $newlines) = @_;
+
+	# Regex taken from perlsyn, with the correction that there's no space
+	# required between the line number and the file name.
+	if ($start->[LOCATION_CHARACTER] == 1) {
+		if ( $Token->isa('PPI::Token::Comment') ) {
+			if (
+				$Token->content =~ m<
+					\A
+					\#      \s*
+					line    \s+
+					(\d+)   \s*
+					(?: (\"?) ([^\"]* [^\s\"]) \2 )?
+					\s*
+					\z
+				>xms
+			) {
+				return $1, ($3 || $start->[LOCATION_LOGICAL_FILE]);
+			}
+		}
+		elsif ( $Token->isa('PPI::Token::Pod') ) {
+			my $content = $Token->content;
+			my $line;
+			my $file = $start->[LOCATION_LOGICAL_FILE];
+			my $end_of_directive;
+			while (
+				$content =~ m<
+					^
+					\#      \s*?
+					line    \s+?
+					(\d+)   (?: (?! \n) \s)*
+					(?: (\"?) ([^\"]*? [^\s\"]) \2 )??
+					\s*?
+					$
+				>xmsg
+			) {
+				($line, $file) = ($1, ( $3 || $file ) );
+				$end_of_directive = pos $content;
+			}
+
+			if (defined $line) {
+				pos $content = $end_of_directive;
+				my $post_directive_newlines =()= $content =~ m< \G [^\n]* \n >xmsg;
+				return $line + $post_directive_newlines - 1, $file;
+			}
+		}
+	}
+
+	return
+		$start->[LOCATION_LOGICAL_LINE] + $newlines,
+		$start->[LOCATION_LOGICAL_FILE];
+}
+
 sub _visual_length {
 	my ($self, $content, $pos) = @_;
 
@@ -697,15 +852,46 @@ Returns a L<PPI::Document::Normalized> object, or C<undef> on error.
 =cut
 
 sub normalized {
-	my $self = shift;
-
 	# The normalization process will utterly destroy and mangle
 	# anything passed to it, so we are going to only give it a
-	# clone of ourself.
-	my $Document = $self->clone or return undef;
+	# clone of ourselves.
+	PPI::Normal->process( $_[0]->clone );
+}
 
-	# Create the normalization object and execute it
-	PPI::Normal->process( $Document );
+=pod
+
+=head1 complete
+
+The C<complete> method is used to determine if a document is cleanly
+structured, all braces are closed, the final statement is
+fully terminated and all heredocs are fully entered.
+
+Returns true if the document is complete or false if not.
+
+=cut
+
+sub complete {
+	my $self = shift;
+
+	# Every structure has to be complete
+	$self->find_any( sub {
+		$_[1]->isa('PPI::Structure')
+		and
+		! $_[1]->complete
+	} )
+	and return '';
+
+	# Strip anything that isn't a statement off the end
+	my @child = $self->children;
+	while ( @child and not $child[-1]->isa('PPI::Statement') ) {
+		pop @child;
+	}
+
+	# We must have at least one statement
+	return '' unless @child;
+
+	# Check the completeness of the last statement
+	return $child[-1]->_complete;
 }
 
 
@@ -717,7 +903,7 @@ sub normalized {
 
 # We are a scope boundary
 ### XS -> PPI/XS.xs:_PPI_Document__scope 0.903+
-sub scope { 1 }
+sub scope() { 1 }
 
 
 
@@ -726,18 +912,6 @@ sub scope { 1 }
 #####################################################################
 # PPI::Element Methods
 
-# Is the document complete.
-# Cascase to the last significant child
-sub complete {
-	my $self  = shift;
-	my $child = $self->schild(-1);
-	return !! (
-		$child
-		and
-		$child->complete
-	);
-}
-
 sub insert_before {
 	return undef;
 	# die "Cannot insert_before a PPI::Document";
@@ -830,11 +1004,11 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-L<PPI>, L<http://ali.as/>
+L<PPI>, L<https://web.archive.org/web/20230911221703/http://ali.as/>
 
 =head1 COPYRIGHT
 
-Copyright 2001 - 2008 Adam Kennedy.
+Copyright 2001 - 2011 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
