@@ -33,12 +33,9 @@ generate the dump content itself.
 =cut
 
 use strict;
-use Params::Util '_INSTANCE';
+use Params::Util qw{_INSTANCE};
 
-use vars qw{$VERSION};
-BEGIN {
-	$VERSION = '1.203';
-}
+our $VERSION = '1.284';
 
 
 
@@ -92,7 +89,7 @@ made much clearer. True/false value, on by default.
 
 Should the dumper show comment tokens. In situations where you have
 a lot of comments, the code can often be made clearer by ignoring
-comment tokens. True/value value, on by default.
+comment tokens. True/false value, on by default.
 
 =item locations
 
@@ -123,7 +120,8 @@ sub new {
 		}, $class;
 
 	# Handle the options
-	my %options = map { lc $_ } @_;
+	my @options = map { lc $_ } @_; # strict hashpairs # https://github.com/Perl-Critic/PPI/issues/201
+	my %options = @options;
 	foreach ( keys %{$self->{display}} ) {
 		if ( exists $options{$_} ) {
 			if ( $_ eq 'indent' ) {
@@ -136,16 +134,6 @@ sub new {
 
 	$self->{indent_string} = join '', (' ' x $self->{display}->{indent});
 
-	# Try to auto-call index_locations. If it failes, turn of locations display
-	if ( $self->{display}->{locations} ) {
-		my $Document = $Element->isa('PPI::Document') ? $Element : $Element->top;
-		if ( $Document->isa('PPI::Document') ) {
-			$Document->index_locations();
-		} else {
-			$self->{display}->{locations} = 0;
-		}
-	}
-        
 	$self;
 }
 
@@ -193,7 +181,7 @@ sub string {
 The C<list> method generates the dump and provides it as a raw
 list, without trailing newlines.
 
-Returns a list or the null list if there is an error while generation
+Returns a list or the null list if there is an error while generating
 the dump.
 
 =cut
@@ -276,8 +264,10 @@ sub _element_string {
 			my $content = $Element->content;
 			$content =~ s/\n/\\n/g;
 			$content =~ s/\t/\\t/g;
+			$content =~ s/\f/\\f/g;
 			$string .= "  \t'$content'";
 		}
+
 	} elsif ( $Element->isa('PPI::Structure') ) {
 		# Add the content
 		if ( $self->{display}->{content} ) {
@@ -308,7 +298,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2001 - 2008 Adam Kennedy.
+Copyright 2001 - 2011 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
