@@ -151,13 +151,11 @@ use Scalar::Util   ();
 use Params::Util   qw{_INSTANCE};
 use PPI::Node      ();
 use PPI::Exception ();
+use PPI::Singletons '%_PARENT';
 
-use vars qw{$VERSION @ISA *_PARENT};
-BEGIN {
-	$VERSION = '1.220';
-	@ISA     = 'PPI::Node';
-	*_PARENT = *PPI::Element::_PARENT;
-}
+our $VERSION = '1.284';
+
+our @ISA = "PPI::Node";
 
 use PPI::Statement::Break          ();
 use PPI::Statement::Compound       ();
