@@ -36,28 +36,37 @@ about the difference between how perl parses Perl "code" and how PPI
 parsers Perl "documents".
 
 "perl" itself (the interpreter) uses a heavily modified lex specification
-to specify it's parsing logic, maintains several types of state as it
-goes, and both tokenizes and lexes AND EXECUTES at the same time.
-In fact, it's provably impossible to use perl's parsing method without
-BEING perl.
+to specify its parsing logic, maintains several types of state as it
+goes, and incrementally tokenizes, lexes AND EXECUTES at the same time.
+
+In fact, it is provably impossible to use perl's parsing method without
+simultaneously executing code. A formal mathematical proof has been
+published demonstrating the method.
 
 This is where the truism "Only perl can parse Perl" comes from.
 
-PPI uses a completely different approach by abandoning the ability to
-provably parse perl, and instead to parse the source as a document, but
-get so close to being perfect that unless you do insanely silly things,
-it can handle it.
+PPI uses a completely different approach by abandoning the (impossible)
+ability to parse Perl the same way that the interpreter does, and instead
+parsing the source as a document, using a document structure independently
+derived from the Perl documentation and approximating the perl interpreter
+interpretation as closely as possible.
 
 It was touch and go for a long time whether we could get it close enough,
 but in the end it turned out that it could be done.
 
-In this approach, the tokenizer C<PPI::Tokenizer> is separate from the
-lexer L<PPI::Lexer>. The job of C<PPI::Tokenizer> is to take pure source
-as a string and break it up into a stream/set of tokens.
+In this approach, the tokenizer C<PPI::Tokenizer> is implemented separately
+from the lexer L<PPI::Lexer>.
 
-The Tokenizer uses a hell of a lot of heuristics, guessing, and cruft,
-supported by a very B<VERY> flexible internal API, but fortunately there's
-not a lot that gets exposed to people using the C<PPI::Tokenizer> itself.
+The job of C<PPI::Tokenizer> is to take pure source as a string and break it
+up into a stream/set of tokens, and contains most of the "black magic" used
+in PPI. By comparison, the lexer implements a relatively straight forward
+tree structure, and has an implementation that is uncomplicated (compared
+to the insanity in the tokenizer at least).
+
+The Tokenizer uses an immense amount of heuristics, guessing and cruft,
+supported by a very B<VERY> flexible internal API, but fortunately it was
+possible to largely encapsulate the black magic, so there is not a lot that
+gets exposed to people using the C<PPI::Tokenizer> itself.
 
 =head1 METHODS
 
@@ -67,25 +76,59 @@ in private methods.
 
 =cut
 
-# Make sure everything we need is loaded, so we
-# don't have to go and load all of PPI.
+# Make sure everything we need is loaded so
+# we don't have to go and load all of PPI.
 use strict;
-use List::MoreUtils ();
-use Params::Util    '_INSTANCE',
-                    '_SCALAR',
-                    '_ARRAY';
+use Params::Util    qw{_INSTANCE _SCALAR0 _ARRAY0};
+use List::Util 1.33 ();
+use PPI::Util       ();
 use PPI::Element    ();
 use PPI::Token      ();
-use PPI::Util       ();
 use PPI::Exception  ();
+use PPI::Exception::ParserRejection ();
+use PPI::Document ();
 
-use vars qw{$VERSION $errstr};
-BEGIN {
-	$VERSION = '1.203';
-	$errstr  = '';
-}
+our $VERSION = '1.284';
 
+# The x operator cannot follow most Perl operators, implying that
+# anything beginning with x following an operator is a word.
+# These are the exceptions.
+my %X_CAN_FOLLOW_OPERATOR = map { $_ => 1 } qw( -- ++ );
 
+# The x operator cannot follow most structure elements, implying that
+# anything beginning with x following a structure element is a word.
+# These are the exceptions.
+my %X_CAN_FOLLOW_STRUCTURE = map { $_ => 1 } qw( } ] \) );
+
+# Something that looks like the x operator but follows a word
+# is usually that word's argument.
+# These are the exceptions.
+# chop, chomp, dump are ambiguous because they can have either parms
+# or no parms.
+my %X_CAN_FOLLOW_WORD = map { $_ => 1 } qw(
+		endgrent
+		endhostent
+		endnetent
+		endprotoent
+		endpwent
+		endservent
+		fork
+		getgrent
+		gethostent
+		getlogin
+		getnetent
+		getppid
+		getprotoent
+		getpwent
+		getservent
+		setgrent
+		setpwent
+		time
+		times
+		wait
+		wantarray
+		__SUB__
+);
 
 
 
@@ -94,7 +137,7 @@ BEGIN {
 
 =pod
 
-=head2 new $source | \@lines | \$source
+=head2 new $file | \@lines | \$source
 
 The main C<new> constructor creates a new Tokenizer object. These
 objects have no configuration parameters, and can only be used once,
@@ -104,75 +147,76 @@ It takes as argument either a normal scalar containing source code,
 a reference to a scalar containing source code, or a reference to an
 ARRAY containing newline-terminated lines of source code.
 
-Returns a new C<PPI::Tokenizer> object on success, or C<undef> on error.
+Returns a new C<PPI::Tokenizer> object on success, or throws a
+L<PPI::Exception> exception on error.
 
 =cut
 
 sub new {
-	my $class = (ref $_[0] ? ref shift : shift)->_clear;
+	my $class = ref($_[0]) || $_[0];
 
 	# Create the empty tokenizer struct
 	my $self = bless {
 		# Source code
-		source         => undef,
-		source_bytes   => undef,
+		source       => undef,
+		source_bytes => undef,
+		document     => undef,
 
 		# Line buffer
-		line           => undef,
-		line_length    => undef,
-		line_cursor    => undef,
-		line_count     => 0,
+		line         => undef,
+		line_length  => undef,
+		line_cursor  => undef,
+		line_count   => 0,
 
 		# Parse state
-		token          => undef,
-		class          => 'PPI::Token::Whitespace',
-		zone           => 'PPI::Token::Whitespace',
+		token        => undef,
+		class        => 'PPI::Token::BOM',
+		zone         => 'PPI::Token::Whitespace',
 
 		# Output token buffer
-		tokens         => [],
-		token_cursor   => 0,
-		token_eof      => 0,
+		tokens       => [],
+		token_cursor => 0,
+		token_eof    => 0,
 
-		# Perl 5 blocks
-		v6             => [],
+		# Perl 6 blocks
+		perl6        => [],
 	}, $class;
 
-	if ( ! defined $_[0] ) {
+	if ( ! defined $_[1] ) {
 		# We weren't given anything
-		return $self->_error( "No source provided to Tokenizer" );
+		PPI::Exception->throw("No source provided to Tokenizer");
 
-	} elsif ( ! ref $_[0] ) {
-		my $source = PPI::Util::_slurp($_[0]);
+	} elsif ( ! ref $_[1] ) {
+		my $source = PPI::Util::_slurp($_[1]);
 		if ( ref $source ) {
 			# Content returned by reference
 			$self->{source} = $$source;
 		} else {
 			# Errors returned as a string
-			return($source);
+			return( $source );
 		}
 
-	} elsif ( _SCALAR($_[0]) ) {
-		$self->{source} = ${shift()};
+	} elsif ( _SCALAR0($_[1]) ) {
+		$self->{source} = ${$_[1]};
 
-	} elsif ( _ARRAY($_[0]) ) {
-		$self->{source} = join "\n", @{shift()};
+	} elsif ( _ARRAY0($_[1]) ) {
+		$self->{source} = join '', map { "\n" } @{$_[1]};
 
 	} else {
 		# We don't support whatever this is
-		return $self->_error( ref($_[0]) . " is not supported as a source provider" );
+		PPI::Exception->throw(ref($_[1]) . " is not supported as a source provider");
 	}
 
-	# Localise newlines
-	$self->{source} =~ s/(?:\015{1,2}\012|\015|\012)/\n/g;
-
 	# We can't handle a null string
 	$self->{source_bytes} = length $self->{source};
-	unless ( $self->{source_bytes} ) {
-		return $self->_error( "Nothing to parse" );
-	}
+	if ( $self->{source_bytes} ) {
+		# Split on local newlines
+		$self->{source} =~ s/(?:\015{1,2}\012|\015|\012)/\n/g;
+		$self->{source} = [ split /(?<=\n)/, $self->{source} ];
 
-	# Split into the line array
-	$self->{source} = [ split /(?<=\n)/, $self->{source} ];
+	} else {
+		$self->{source} = [ ];
+	}
 
 	### EVIL
 	# I'm explaining this earlier than I should so you can understand
@@ -192,7 +236,9 @@ sub new {
 	# once the tokenizer hits end of file, it examines the last token to
 	# manually either remove the ' ' token, or chop it off the end of
 	# a longer one in which the space would be valid.
-	if ( List::MoreUtils::any { /^__(?:DATA|END)__\s*$/ } @{$self->{source}} ) {
+	if ( List::Util::any { /^__(?:DATA|END)__\s*$/ } @{$self->{source}} ) {
+		$self->{source_eof_chop} = '';
+	} elsif ( ! defined $self->{source}->[0] ) {
 		$self->{source_eof_chop} = '';
 	} elsif ( $self->{source}->[-1] =~ /\s$/ ) {
 		$self->{source_eof_chop} = '';
@@ -204,6 +250,11 @@ sub new {
 	$self;
 }
 
+sub _document {
+	my $self = shift;
+	return @_ ? $self->{document} = shift : $self->{document};
+}
+
 
 
 
@@ -258,7 +309,8 @@ sub get_token {
 		# No token, we need to get some more
 		while ( $line_rv = $self->_process_next_line ) {
 			# If there is something in the buffer, return it
-			if ( my $token = $self->{tokens}->[ $self->{token_cursor} ] ) {
+			# The defined() prevents a ton of calls to PPI::Util::TRUE
+			if ( defined( my $token = $self->{tokens}->[ $self->{token_cursor} ] ) ) {
 				$self->{token_cursor}++;
 				return $token;
 			}
@@ -267,11 +319,12 @@ sub get_token {
 	};
 	if ( $@ ) {
 		if ( _INSTANCE($@, 'PPI::Exception') ) {
-			return $self->_error( $@->message );
+			$@->throw;
+		} else {
+			my $errstr = $@;
+			$errstr =~ s/^(.*) at line .+$/$1/;
+			PPI::Exception->throw( $errstr );
 		}
-		my $errstr = $@;
-		$errstr =~ s/^(.*) at line .+$/$1/;
-		return $self->_error( $errstr );
 	} elsif ( $rv ) {
 		return $rv;
 	}
@@ -304,9 +357,8 @@ It should be noted that C<all_tokens> does B<NOT> interfere with the
 use of the Tokenizer object as an iterator (does not modify the token
 cursor) and use of the two different mechanisms can be mixed safely.
 
-Returns a reference to an ARRAY of L<PPI::Token> objects on success,
-C<0> in the special case that the file/string contains NO tokens at
-all, or C<undef> on error.
+Returns a reference to an ARRAY of L<PPI::Token> objects on success
+or throws an exception on error.
 
 =cut
 
@@ -315,32 +367,35 @@ sub all_tokens {
 
 	# Catch exceptions and return undef, so that we
 	# can start to convert code to exception-based code.
-	eval {
+	my $ok = eval {
 		# Process lines until we get EOF
 		unless ( $self->{token_eof} ) {
 			my $rv;
 			while ( $rv = $self->_process_next_line ) {}
-			return $self->_error( "Error while processing source" ) unless defined $rv;
+			unless ( defined $rv ) {
+				PPI::Exception->throw("Error while processing source");
+			}
 
 			# Clean up the end of the tokenizer
 			$self->_clean_eof;
 		}
+		1;
 	};
-	if ( $@ ) {
+	if ( !$ok ) {
 		my $errstr = $@;
 		$errstr =~ s/^(.*) at line .+$/$1/;
-		return $self->_error( $errstr );
+		PPI::Exception->throw( $errstr );
 	}
 
 	# End of file, return a copy of the token array.
-	@{ $self->{tokens} } ? [ @{$self->{tokens}} ] : 0;
+	return [ @{$self->{tokens}} ];
 }
 
 =pod
 
 =head2 increment_cursor
 
-Although exposed as a public method, C<increment_method> is implemented
+Although exposed as a public method, C<increment_cursor> is implemented
 for expert use only, when writing lexers or other components that work
 directly on token streams.
 
@@ -362,7 +417,7 @@ sub increment_cursor {
 
 =head2 decrement_cursor
 
-Although exposed as a public method, C<decrement_method> is implemented
+Although exposed as a public method, C<decrement_cursor> is implemented
 for expert use only, when writing lexers or other components that work
 directly on token streams.
 
@@ -414,7 +469,6 @@ sub _get_line {
 # Fetches the next line, ready to process
 # Returns 1 on success
 # Returns 0 on EOF
-# Returns undef on error
 sub _fill_line {
 	my $self   = shift;
 	my $inscan = shift;
@@ -485,13 +539,13 @@ sub _process_next_line {
 
 		# Defined but false means next line
 		return 1 if defined $rv;
-		throw PPI::Exception("Error at line $self->{line_count}");
+		PPI::Exception->throw("Error at line $self->{line_count}");
 	}
 
 	# If we can't deal with the entire line, process char by char
 	while ( $rv = $self->_process_next_char ) {}
 	unless ( defined $rv ) {
-		throw PPI::Exception("Error at line $self->{line_count}, character $self->{line_cursor}");
+		PPI::Exception->throw("Error at line $self->{line_count}, character $self->{line_cursor}");
 	}
 
 	# Trigger any action that needs to happen at the end of a line
@@ -502,108 +556,9 @@ sub _process_next_line {
 		return $self->_clean_eof;
 	}
 
-	# If we have any entries in the rawinput queue process it.
-	# This should happen at the END of this line, not the beginning of
-	# the next one, because Tokenizer->get_token, may retrieve the RawInput
-	# terminator and process it before we get a chance to rebless it from
-	# a bareword or a string that it was originally. Note also that
-	# _handle_raw_input has the same return conditions as this method.
-	$self->{rawinput_queue} ? $self->_handle_raw_input : 1;
+	return 1;
 }
 
-# Read in raw input from the source
-# Returns 1 on success
-# Returns 0 on EOF
-# Returns undef on error
-sub _handle_raw_input {
-	my $self = shift;
-
-	# Is there a half finished token?
-	if ( defined $self->{token} ) {
-		if ( ref($self->{token}) eq 'PPI::Token::Whitespace' ) {
-			# Finish the whitespace token
-			$self->_finalize_token;
-		} else {
-			# This is just a little too complicated to tokenize.
-			# The Perl interpretor has the luxury of being able to
-			# destructively consume the input. We don't... so having
-			# a token that SPANS OVER a raw input is just silly, and
-			# too complicated for this little parser to turn back
-			# into something useful.
-			return $self->_error( "The code is too crufty for the tokenizer.\n"
-				. "Cannot have tokens that span across rawinput lines." );
-		}
-	}
-
-	while ( scalar @{$self->{rawinput_queue}} ) {
-		# Find the rawinput operator and terminator
-		my $position = shift @{$self->{rawinput_queue}};
-		my $operator = $self->{tokens}->[ $position ];
-		my $terminator = $self->{tokens}->[ $position + 1 ];
-
-		# Handle a whitespace gap between the operator and terminator
-		if ( ref($terminator) eq 'PPI::Token::Whitespace' ) {
-			$terminator = $self->{tokens}->[ $position + 2 ];
-		}
-
-		# Check the terminator, and get the termination string
-		my $tString;
-		if ( ref($terminator) eq 'PPI::Token::Word' ) {
-			$tString = $terminator->{content};
-		} elsif ( ref($terminator) =~ /^PPI::Token::Quote::(Single|Double)$/ ) {
-			$tString = $terminator->string;
-			return undef unless defined $tString;
-		} else {
-			return $self->_error( "Syntax error. The raw input << operator must be followed by a bare word, or a single or double quoted string" );
-		}
-		$tString .= "\n";
-
-		# Change the class of the terminator token to the appropriate one
-		$terminator->set_class( 'RawInput::Terminator' );
-
-		# Create the token
-		my $rawinput = PPI::Token::RawInput::String->new( '' ) or return undef;
-		$rawinput->{endString} = $tString;
-
-		# Add some extra links, so these will know where its other parts are
-		$rawinput->{_operator} = $operator;
-		$operator->{_string}   = $rawinput;
-
-		# Start looking at lines, and pull new ones until we find the
-		# termination string.
-		my $rv;
-		while ( $rv = $self->_fill_line ) {
-			# Add to the token
-			$rawinput->{content} .= $self->{line};
-
-			# Does the line match the termination string
-			if ( $self->{line} eq $tString ) {
-				# Done
-				push @{ $self->{tokens} }, $rawinput;
-				last;
-			}
-		}
-
-		# End of this rawinput
-		next if $rv;
-
-		# Pass on any error
-		return undef unless defined $rv;
-
-		# End of file. We are a bit more lenient on errors, so
-		# we will let this slip by without mention. In fact, it may
-		# well actually be legal.
-
-		# Finish the token
-		push @{ $self->{tokens} }, $rawinput if $rawinput->{content} ne '';
-		return 0;
-	}
-
-	# Clean up and return true
-	delete $self->{rawinput_queue};
-
-	1;
-}
 
 
 
@@ -612,24 +567,24 @@ sub _handle_raw_input {
 # Per-character processing methods
 
 # Process on a per-character basis.
-# Note that due the the high number of times this gets
+# Note that due the high number of times this gets
 # called, it has been fairly heavily in-lined, so the code
 # might look a bit ugly and duplicated.
 sub _process_next_char {
 	my $self = shift;
 
 	### FIXME - This checks for a screwed up condition that triggers
-	###         several warnings, amoungst other things.
+	###         several warnings, amongst other things.
 	if ( ! defined $self->{line_cursor} or ! defined $self->{line_length} ) {
 		# $DB::single = 1;
 		return undef;
 	}
 
-	# Increment the counter and check for end of line
-	return 0 if ++$self->{line_cursor} >= $self->{line_length};
+    $self->{line_cursor}++;
+    return 0 if $self->_at_line_end;
 
 	# Pass control to the token class
-        my $result;
+	my $result;
 	unless ( $result = $self->{class}->__TOKENIZER__on_char( $self ) ) {
 		# undef is error. 0 is "Did stuff ourself, you don't have to do anything"
 		return defined $result ? 1 : undef;
@@ -645,7 +600,7 @@ sub _process_next_char {
 		if ( defined $self->{token} ) {
 			$self->{token}->{content} .= $char;
 		} else {
-			$self->{token} = $self->{class}->new( $char ) or return undef;
+			defined($self->{token} = $self->{class}->new($char)) or return undef;
 		}
 
 		return 1;
@@ -660,12 +615,17 @@ sub _process_next_char {
 		$self->{token}->{content} .= $char;
 	} else {
 		# Same class, but no current
-		$self->{token} = $self->{class}->new( $char ) or return undef;
+		defined($self->{token} = $self->{class}->new($char)) or return undef;
 	}
 
 	1;
 }
 
+sub _at_line_end {
+    my ($self) = @_;
+    return $self->{line_cursor} >= $self->{line_length};
+}
+
 
 
 
@@ -677,7 +637,7 @@ sub _process_next_char {
 # Returns the resulting parse class as a convenience.
 sub _finalize_token {
 	my $self = shift;
-	return $self->{class} unless $self->{token};
+	return $self->{class} unless defined $self->{token};
 
 	# Add the token to the token buffer
 	push @{ $self->{tokens} }, $self->{token};
@@ -688,6 +648,7 @@ sub _finalize_token {
 }
 
 # Creates a new token and sets it in the tokenizer
+# The defined() in here prevent a ton of calls to PPI::Util::TRUE
 sub _new_token {
 	my $self = shift;
 	# throw PPI::Exception() unless @_;
@@ -695,10 +656,10 @@ sub _new_token {
 		? shift : 'PPI::Token::' . shift;
 
 	# Finalize any existing token
-	$self->_finalize_token if $self->{token};
+	$self->_finalize_token if defined $self->{token};
 
 	# Create the new token and update the parse class
-	$self->{token} = $class->new($_[0]) or throw PPI::Exception();
+	defined($self->{token} = $class->new($_[0])) or PPI::Exception->throw;
 	$self->{class} = $class;
 
 	1;
@@ -715,7 +676,7 @@ sub _clean_eof {
 	# Find the last token, and if it has no content, kill it.
 	# There appears to be some evidence that such "null tokens" are
 	# somehow getting created accidentally.
-	my $last_token = $self->{tokens}->[ $#{$self->{tokens}} ];
+	my $last_token = $self->{tokens}->[ -1 ];
 	unless ( length $last_token->{content} ) {
 		pop @{$self->{tokens}};
 	}
@@ -723,7 +684,7 @@ sub _clean_eof {
 	# Now, if the last character of the last token is a space we added,
 	# chop it off, deleting the token if there's nothing else left.
 	if ( $self->{source_eof_chop} ) {
-		$last_token = $self->{tokens}->[ $#{$self->{tokens}} ];
+		$last_token = $self->{tokens}->[ -1 ];
 		$last_token->{content} =~ s/ $//;
 		unless ( length $last_token->{content} ) {
 			# Popping token
@@ -746,52 +707,55 @@ sub _clean_eof {
 # Utility Methods
 
 # Context
-sub _last_token { $_[0]->{tokens}->[-1] }
+sub _last_token {
+	$_[0]->{tokens}->[-1];
+}
+
 sub _last_significant_token {
-	my $self = shift;
+	my $self   = shift;
 	my $cursor = $#{ $self->{tokens} };
 	while ( $cursor >= 0 ) {
 		my $token = $self->{tokens}->[$cursor--];
 		return $token if $token->significant;
 	}
-
-	# Nothing...
-	PPI::Token::Whitespace->null;
+	return;
 }
 
 # Get an array ref of previous significant tokens.
 # Like _last_significant_token except it gets more than just one token
-# Returns array ref on success.
-# Returns 0 on not enough tokens
+# Returns array with 0 to x entries
 sub _previous_significant_tokens {
 	my $self   = shift;
 	my $count  = shift || 1;
 	my $cursor = $#{ $self->{tokens} };
 
-	my ($token, @tokens);
+	my @tokens;
 	while ( $cursor >= 0 ) {
-		$token = $self->{tokens}->[$cursor--];
-		if ( $token->significant ) {
-			push @tokens, $token;
-			return \@tokens if scalar @tokens >= $count;
-		}
+		my $token = $self->{tokens}->[$cursor--];
+		next if not $token->significant;
+		push @tokens, $token;
+		last if @tokens >= $count;
 	}
 
-	# Pad with empties
-	foreach ( 1 .. ($count - scalar @tokens) ) {
-		push @tokens, PPI::Token::Whitespace->null;
-	}
-
-	\@tokens;
+	return @tokens;
 }
 
 my %OBVIOUS_CLASS = (
-	'PPI::Token::Symbol'     => 'operator',
-	'PPI::Token::Magic'      => 'operator',
-	'PPI::Token::Number'     => 'operator',
-	'PPI::Token::ArrayIndex' => 'operator',
-	'PPI::Token::Quote'      => 'operator',
+	'PPI::Token::Symbol'              => 'operator',
+	'PPI::Token::Magic'               => 'operator',
+	'PPI::Token::Number'              => 'operator',
+	'PPI::Token::ArrayIndex'          => 'operator',
+	'PPI::Token::Quote::Double'       => 'operator',
+	'PPI::Token::Quote::Interpolate'  => 'operator',
+	'PPI::Token::Quote::Literal'      => 'operator',
+	'PPI::Token::Quote::Single'       => 'operator',
+	'PPI::Token::QuoteLike::Backtick' => 'operator',
+	'PPI::Token::QuoteLike::Command'  => 'operator',
+	'PPI::Token::QuoteLike::Readline' => 'operator',
+	'PPI::Token::QuoteLike::Regexp'   => 'operator',
+	'PPI::Token::QuoteLike::Words'    => 'operator',
 );
+
 my %OBVIOUS_CONTENT = (
 	'(' => 'operand',
 	'{' => 'operand',
@@ -800,15 +764,19 @@ my %OBVIOUS_CONTENT = (
 	'}' => 'operator',
 );
 
-# Try to determine operator/operand context, is possible.
+
+my %USUALLY_FORCES = map { $_ => 1 } qw( sub package use no );
+
+# Try to determine operator/operand context, if possible.
 # Returns "operator", "operand", or "" if unknown.
 sub _opcontext {
 	my $self   = shift;
-	my $tokens = $self->_previous_significant_tokens(1);
-	my $p0     = $tokens->[0];
+	my @tokens = $self->_previous_significant_tokens(1);
+	my $p0     = $tokens[0];
+	return '' if not $p0;
+	my $c0     = ref $p0;
 
 	# Map the obvious cases
-	my $c0 = ref $p0;
 	return $OBVIOUS_CLASS{$c0}   if defined $OBVIOUS_CLASS{$c0};
 	return $OBVIOUS_CONTENT{$p0} if defined $OBVIOUS_CONTENT{$p0};
 
@@ -822,39 +790,91 @@ sub _opcontext {
 	return ''
 }
 
+# Assuming we are currently parsing the word 'x', return true
+# if previous tokens imply the x is an operator, false otherwise.
+sub _current_x_is_operator {
+	my ( $self ) = @_;
+	return if !@{$self->{tokens}};
 
+	my ($prev, $prevprev) = $self->_previous_significant_tokens(2);
+	return if !$prev;
 
+	return !$self->__current_token_is_forced_word if $prev->isa('PPI::Token::Word');
 
-
-#####################################################################
-# Error Handling
-
-# Set the error message
-sub _error {
-	$errstr = $_[1];
-	undef;
+	return (!$prev->isa('PPI::Token::Operator') || $X_CAN_FOLLOW_OPERATOR{$prev})
+		&& (!$prev->isa('PPI::Token::Structure') || $X_CAN_FOLLOW_STRUCTURE{$prev})
+		&& !$prev->isa('PPI::Token::Label')
+	;
 }
 
-# Clear the error message.
-# Returns the object as a convenience.
-sub _clear {
-	$errstr = '';
-	$_[0];
+
+# Assuming we are at the end of parsing the current token that could be a word,
+# a wordlike operator, or a version string, try to determine whether context
+# before or after it forces it to be a bareword. This method is only useful
+# during tokenization.
+sub __current_token_is_forced_word {
+	my ( $t, $word ) = @_;
+
+	# Check if forced by preceding tokens.
+
+	my ( $prev, $prevprev ) = $t->_previous_significant_tokens(2);
+	if ( !$prev ) {
+		pos $t->{line} = $t->{line_cursor};
+	}
+	else {
+		my $content = $prev->{content};
+
+		# We are forced if we are a method name.
+		# '->' will always be an operator, so we don't check its type.
+		return 1 if $content eq '->';
+
+		# If we are contained in a pair of curly braces, we are probably a
+		# forced bareword hash key. '{' is never a word or operator, so we
+		# don't check its type.
+		pos $t->{line} = $t->{line_cursor};
+		return 1 if $content eq '{' and $t->{line} =~ /\G\s*\}/gc;
+
+		# sub, package, use, and no all indicate that what immediately follows
+		# is a word not an operator or (in the case of sub and package) a
+		# version string.  However, we don't want to be fooled by 'package
+		# package v10' or 'use no v10'. We're a forced package unless we're
+		# preceded by 'package sub', in which case we're a version string.
+		# We also have to make sure that the sub/package/etc doing the forcing
+		# is not a method call.
+		if( $USUALLY_FORCES{$content}) {
+			return if defined $word and $word =~ /^v[0-9]+$/ and ( $content eq "use" or $content eq "no" );
+			return 1 if not $prevprev;
+			return 1 if not $USUALLY_FORCES{$prevprev->content} and $prevprev->content ne '->';
+			return;
+		}
+	}
+	# pos on $t->{line} is guaranteed to be set at this point.
+
+	# Check if forced by following tokens.
+
+	# If the word is followed by => it is probably a word, not a regex.
+	return 1 if $t->{line} =~ /\G\s*=>/gc;
+
+	# Otherwise we probably aren't forced
+	return '';
 }
 
-=pod
+sub _current_token_has_signatures_active {
+	my ($t) = @_;
 
-=head2 errstr
+	# Get at least the three previous significant tokens, and extend the
+	# retrieval range to include at least one token that can walk the
+	# already generated tree. (i.e. has a parent)
+	my ( $tokens_to_get, @tokens ) = (3);
+	while ( !@tokens or ( $tokens[-1] and !$tokens[-1]->parent ) ) {
+		@tokens = $t->_previous_significant_tokens($tokens_to_get);
+		last if @tokens < $tokens_to_get;
+		$tokens_to_get++;
+	}
 
-For any error that occurs, you can use the C<errstr>, as either
-a static or object method, to access the error message.
-
-If no error occurs for any particular action, C<errstr> will return false.
-
-=cut
-
-sub errstr {
-	$errstr;
+	my ($closest_parented_token) = grep $_->parent, @tokens;
+	$closest_parented_token ||= $t->_document || $t->_document(PPI::Document->new);
+	return $closest_parented_token->presumed_features->{signatures}, @tokens;
 }
 
 1;
@@ -865,7 +885,7 @@ sub errstr {
 
 =head2 How the Tokenizer Works
 
-Understanding the Tokenizer is not for the feint-hearted. It is by far
+Understanding the Tokenizer is not for the faint-hearted. It is by far
 the most complex and twisty piece of perl I've ever written that is actually
 still built properly and isn't a terrible spaghetti-like mess. In fact, you
 probably want to skip this section.
@@ -896,7 +916,7 @@ called in whatever token class we are currently in, which will examine the
 character at the current position, and handle it.
 
 As the handler methods in the various token classes are called, they
-build up a output token array for the source code.
+build up an output token array for the source code.
 
 Various parts of the Tokenizer use look-ahead, arbitrary-distance
 look-behind (although currently the maximum is three significant tokens),
@@ -1092,7 +1112,7 @@ Adam Kennedy E<lt>adamk@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2001 - 2008 Adam Kennedy.
+Copyright 2001 - 2011 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
